package SHARYANTO::Utils;

our $VERSION = '0.45'; # VERSION

1;
# ABSTRACT: SHARYANTO's temporary namespace for various routines


=pod

=head1 NAME

SHARYANTO::Utils - SHARYANTO's temporary namespace for various routines

=head1 VERSION

version 0.45

=head1 FAQ

=head2 What is this?

This distribution is a heterogenous collection of modules that will eventually
have their own proper distributions. See L<SHARYANTO> about this temporary
namespace.

=head1 AUTHOR

Steven Haryanto <stevenharyanto@gmail.com>

=head1 COPYRIGHT AND LICENSE

This software is copyright (c) 2013 by Steven Haryanto.

This is free software; you can redistribute it and/or modify it under
the same terms as the Perl 5 programming language system itself.

=head1 FUNCTIONS

=cut


__END__

